@@ -32,14 +32,66 @@
 #include <utils/qtcassert.h>
 
 #include <QCryptographicHash>
+#include <QElapsedTimer>
 #include <QLoggingCategory>
 
 #include <algorithm>
 
 static Q_LOGGING_CATEGORY(importsLog, "qtc.qmljs.imports", QtWarningMsg)
+static Q_LOGGING_CATEGORY(importsBenchmark, "qtc.qmljs.imports.benchmark", QtWarningMsg)
+
+
+class ImportsBenchmarker
+{
+public:
+    ImportsBenchmarker(const QString &functionName)
+        : m_functionName(functionName)
+    {
+        m_timer.start();
+    }
+
+    ~ImportsBenchmarker()
+    {
+        if (importsBenchmark().isDebugEnabled()) {
+            qCDebug(importsBenchmark).noquote().nospace() << m_functionName << " executed nPossibleExports: " << nPossibleExports << " in " << m_timer.elapsed() << "ms";
+        }
+    }
+    int nPossibleExports = 0;
+
+    QElapsedTimer m_timer;
+    QString m_functionName;
+};
 
 namespace QmlJS {
 
+/*
+ which languages might be imported in this context
+ */
+static bool languageIsCompatible(Dialect contextLanguage, Dialect importLanguage)
+{
+    if (importLanguage == Dialect::AnyLanguage && contextLanguage != Dialect::NoLanguage)
+        return true;
+    switch (contextLanguage.dialect()) {
+    case Dialect::JavaScript:
+    case Dialect::Json:
+    case Dialect::QmlProject:
+    case Dialect::QmlQbs:
+    case Dialect::QmlTypeInfo:
+        return contextLanguage == importLanguage;
+    case Dialect::Qml:
+        return importLanguage == Dialect::Qml || importLanguage == Dialect::QmlQtQuick2 || importLanguage == Dialect::JavaScript;
+    case Dialect::QmlQtQuick2:
+    case Dialect::QmlQtQuick2Ui:
+        return importLanguage == Dialect::Qml || importLanguage == Dialect::QmlQtQuick2 || importLanguage == Dialect::QmlQtQuick2Ui
+            || importLanguage == Dialect::JavaScript;
+    case Dialect::AnyLanguage:
+        return true;
+    case Dialect::NoLanguage:
+        break;
+    }
+    return false;
+}
+
 ImportKind::Enum toImportKind(ImportType::Enum type)
 {
     switch (type) {
@@ -119,8 +171,7 @@ ImportKey::ImportKey(const ImportInfo &info)
     , majorVersion(info.version().majorVersion())
     , minorVersion(info.version().minorVersion())
 {
-    splitPath = QFileInfo(info.path()).canonicalFilePath().split(QLatin1Char('/'),
-                                                                 QString::KeepEmptyParts);
+    splitPath = QFileInfo(info.path()).canonicalFilePath().split('/');
 }
 
 ImportKey::ImportKey(ImportType::Enum type, const QString &path, int majorVersion, int minorVersion)
@@ -271,13 +322,13 @@ ImportMatchStrength ImportKey::matchImport(const ImportKey &o, const ViewerConte
     if (dirToFile)
         --lenPath1;
     int iSelector = 0;
-    int nSelectors = vContext.selectors.size();
+    const int nSelectors = vContext.selectors.size();
     while (iPath1 < lenPath1) {
         if (lenPath2 - iPath2 > lenPath1 - iPath1)
             return ImportMatchStrength();
-        QString p1 = splitPath.at(iPath1);
+        const QString p1 = splitPath.at(iPath1);
         if (iPath2 < lenPath2) {
-            QString p2 = splitPath.at(iPath2);
+            const QString p2 = o.splitPath.at(iPath2);
             if (p1 == p2) {
                 ++iPath1;
                 ++iPath2;
@@ -285,24 +336,24 @@ ImportMatchStrength ImportKey::matchImport(const ImportKey &o, const ViewerConte
             }
         }
         if (!p1.startsWith(QLatin1Char('+')))
-            return QList<int>();
-        QStringRef selectorAtt(&p1, 1, p1.size()-1);
+            return ImportMatchStrength();
+        const QStringView selectorAtt(p1.constData() + 1, p1.size() - 1);
         while (iSelector < nSelectors) {
             if (selectorAtt == vContext.selectors.at(iSelector))
                 break;
             ++iSelector;
         }
         if (iSelector == nSelectors)
-            return QList<int>();
+            return ImportMatchStrength();
         res << (nSelectors - iSelector);
         ++iSelector;
         ++iPath1;
     }
     if (iPath2 != lenPath2)
-        return QList<int>();
+        return ImportMatchStrength();
     if (res.isEmpty())
         res << 0;
-    return  ImportMatchStrength(res);
+    return ImportMatchStrength(res);
 }
 
 int ImportKey::compare(const ImportKey &other) const
@@ -493,19 +544,6 @@ bool Export::visibleInVContext(const ViewerContext &vContext) const
     return pathRequired.isEmpty() || vContext.paths.contains(pathRequired);
 }
 
-bool operator ==(const Export &i1, const Export &i2)
-{
-    return i1.exportName == i2.exportName
-            && i1.pathRequired == i2.pathRequired
-            && i1.intrinsic == i2.intrinsic
-            && i1.typeName == i2.typeName;
-}
-
-bool operator !=(const Export &i1, const Export &i2)
-{
-    return !(i1 == i2);
-}
-
 CoreImport::CoreImport() : language(Dialect::Qml) { }
 
 CoreImport::CoreImport(const QString &importId, const QList<Export> &possibleExports,
@@ -582,14 +620,16 @@ ImportDependencies::~ImportDependencies()
 
 void ImportDependencies::filter(const ViewerContext &vContext)
 {
+    ImportsBenchmarker benchMark("filter()");
     QMap<QString, CoreImport> newCoreImports;
     QMap<ImportKey, QStringList> newImportCache;
     bool hasChanges = false;
     for (auto j = m_coreImports.cbegin(), end = m_coreImports.cend(); j != end; ++j) {
         const CoreImport &cImport = j.value();
-        if (vContext.languageIsCompatible(cImport.language)) {
+        if (languageIsCompatible(vContext.language, cImport.language)) {
             QList<Export> newExports;
             foreach (const Export &e, cImport.possibleExports) {
+                ++benchMark.nPossibleExports;
                 if (e.visibleInVContext(vContext)) {
                     newExports.append(e);
                     QStringList &candidateImports = newImportCache[e.exportName];
@@ -627,6 +667,7 @@ void ImportDependencies::iterateOnCandidateImports(
         std::function<bool (const ImportMatchStrength &,const Export &,const CoreImport &)>
         const &iterF) const
 {
+    ImportsBenchmarker benchMark("iterateOnCandidateImports()");
     switch (key.type) {
     case ImportType::Directory:
     case ImportType::QrcDirectory:
@@ -637,8 +678,9 @@ void ImportDependencies::iterateOnCandidateImports(
         const QStringList imp = m_importCache.value(key.flatKey());
         foreach (const QString &cImportName, imp) {
             CoreImport cImport = coreImport(cImportName);
-            if (vContext.languageIsCompatible(cImport.language)) {
+            if (languageIsCompatible(vContext.language, cImport.language)) {
                 foreach (const Export e, cImport.possibleExports) {
+                    ++benchMark.nPossibleExports;
                     if (e.visibleInVContext(vContext)) {
                         ImportMatchStrength m = e.exportName.matchImport(key, vContext);
                         if (m.hasMatch()) {
@@ -659,8 +701,9 @@ void ImportDependencies::iterateOnCandidateImports(
         if (c == ImportKey::SameDir) {
             foreach (const QString &cImportName, lb.value()) {
                 CoreImport cImport = coreImport(cImportName);
-                if (vContext.languageIsCompatible(cImport.language)) {
+                if (languageIsCompatible(vContext.language, cImport.language)) {
                     foreach (const Export e, cImport.possibleExports) {
+                        ++benchMark.nPossibleExports;
                         if (e.visibleInVContext(vContext)) {
                             ImportMatchStrength m = e.exportName.matchImport(key, vContext);
                             if (m.hasMatch()) {
@@ -783,13 +826,13 @@ void ImportDependencies::addExport(const QString &importId, const ImportKey &imp
     if (!m_coreImports.contains(importId)) {
         CoreImport newImport(importId);
         newImport.language = Dialect::AnyLanguage;
-        newImport.possibleExports.append(Export(importKey, requiredPath, false, typeName));
+        newImport.addPossibleExport(Export(importKey, requiredPath, false, typeName));
         m_coreImports.insert(newImport.importId, newImport);
         m_importCache[importKey].append(importId);
         return;
     }
     CoreImport &importValue = m_coreImports[importId];
-    importValue.possibleExports.append(Export(importKey, requiredPath, false, typeName));
+    importValue.addPossibleExport(Export(importKey, requiredPath, false, typeName));
     m_importCache[importKey].append(importId);
     qCDebug(importsLog) << "added export "<< importKey.toString() << " for id " <<importId
                         << " (" << requiredPath << ")";
@@ -826,6 +869,8 @@ void ImportDependencies::iterateOnLibraryImports(
                             const Export &,
                             const CoreImport &)> const &iterF) const
 {
+    ImportsBenchmarker benchMark("iterateOnLibraryImports()");
+
     typedef QMap<ImportKey, QStringList>::const_iterator iter_t;
     ImportKey firstLib;
     firstLib.type = ImportType::Library;
@@ -835,8 +880,9 @@ void ImportDependencies::iterateOnLibraryImports(
         qCDebug(importsLog) << "libloop:" << i.key().toString() << i.value();
         foreach (const QString &cImportName, i.value()) {
             CoreImport cImport = coreImport(cImportName);
-            if (vContext.languageIsCompatible(cImport.language)) {
+            if (languageIsCompatible(vContext.language, cImport.language)) {
                 foreach (const Export &e, cImport.possibleExports) {
+                    ++benchMark.nPossibleExports;
                     if (e.visibleInVContext(vContext) && e.exportName.type == ImportType::Library) {
                         ImportMatchStrength m = e.exportName.matchImport(i.key(), vContext);
                         if (m.hasMatch()) {
@@ -860,6 +906,7 @@ void ImportDependencies::iterateOnSubImports(
                             const Export &,
                             const CoreImport &)> const &iterF) const
 {
+    ImportsBenchmarker benchMark("iterateOnSubImports()");
     typedef QMap<ImportKey, QStringList>::const_iterator iter_t;
     iter_t i = m_importCache.lowerBound(baseKey);
     iter_t end = m_importCache.constEnd();
@@ -869,8 +916,9 @@ void ImportDependencies::iterateOnSubImports(
             break;
         foreach (const QString &cImportName, i.value()) {
             CoreImport cImport = coreImport(cImportName);
-            if (vContext.languageIsCompatible(cImport.language)) {
+            if (languageIsCompatible(vContext.language, cImport.language)) {
                 foreach (const Export &e, cImport.possibleExports) {
+                    ++benchMark.nPossibleExports;
                     if (e.visibleInVContext(vContext)) {
                         ImportMatchStrength m = e.exportName.matchImport(i.key(), vContext);
                         if (m.hasMatch()) {
